@@ -159,7 +159,7 @@ class TaskListView(SwaggerView):
             "name": "state",
             "in": "query",
             "type": "string",
-            "enum": ["submitted", "waiting", "done", "any", "failed", "running"],
+            "enum": ["submitted", "waiting", "done", "any", "failed", "running", "corrupt"],
             "required": False,
             "default": "any",
         },
@@ -449,20 +449,33 @@ class WorkerDataAssertFact(SwaggerView):
         
         logger.info("storing object %s size %s Mb in dag-motivated bucket: %s", dag[-1], len(data_json)/1024./1024, dag_bucket)
 
-        try:
-            open
+        attempts_left = 1
+        bucket = None
 
-            bucket = odakb.datalake.store(
-                        dict(dag=dag, data=data),
-                        bucket_name=dag_bucket,
-                    )
+        package = dict(dag=dag, data=data)
+        while bucket is None:
 
-            logger.info("returned bucket %s", bucket)
+            try:
+                bucket = odakb.datalake.store(
+                            package,
+                            bucket_name=dag_bucket,
+                        )
 
-            assert bucket == dag_bucket
-        except Exception as e:
-            logger.error("unable to store bucket %s in datalake: %s", dag_bucket, e)
-            raise
+                logger.info("returned bucket %s", bucket)
+
+                assert bucket == dag_bucket
+            except Exception as e:
+                logger.error("unable to store bucket %s in datalake: %s", dag_bucket, e)
+                if attempts_left > 0:
+                    logger.warning("will retry bucket store, attempts left %s", attempts_left)
+                    attempts_left -= 1
+
+                    time.sleep(5)
+                else:
+                    fn = f"/tmp/failed-bucket-{dag_bucket}.json"
+                    json.dump(package, open(fn, "w"))
+                    logger.error("stored debug bucket to %s", fn)
+                    raise
 
         logger.info("succesfully returning!")
 
@@ -526,7 +539,7 @@ class WorkerDataConsultFact(SwaggerView):
         
         #TODO: logtask?
 
-        logger.error("return_data %s %s", return_data,type( return_data))
+        #logger.error("return_data %s %s", return_data,type( return_data))
 
         if odakb.datalake.exists(dag_bucket):
             if return_data:
@@ -968,6 +981,7 @@ class WorkerQuestion(SwaggerView):
                         task_entry
                     )
         except Exception as e:
+            logger.error("unable to insert task: %s, task_data: %s", e, task_data)
             r = jsonify(
                         {"exception": "unable to insert"}
                     )
@@ -1203,6 +1217,39 @@ app.add_url_rule(
           methods=['POST']
 )
 
+@app.route("/tasks/delete/<string:scope>/<string:selector>")
+def tasks_delete(scope, selector):
+    """
+    ---
+    operationId: 'delete'
+    parameters:
+    - name: 'queue'
+      in: 'query'
+      required: false
+      type: 'string'
+
+    - name: 'scope'
+      in: 'path'
+      enum: ['state', 'task']
+      required: true 
+      type: 'string'
+
+    - name: 'selector'
+      in: 'path'
+      required: True
+      type: 'string'
+
+    responses:
+        200: 
+            description: 'entries purged'
+    """
+
+    #queue = dqueue.core.Queue(queue)
+    n = tools.delete(scope, selector)
+    return jsonify(
+            nentries=n
+        )
+
 @app.route("/tasks/resubmit/<string:scope>/<string:selector>")
 def tasks_resubmit(scope, selector):
     """
