@@ -393,6 +393,17 @@ def question(obj, task_data):
     r = obj['queue'].put(j_task_data)
     print(colored("questioned:", "green"), task_data, ":", r)
 
+@cli.command()
+@click.option('-s', '--scope-selector', default="state:all")
+@click.pass_obj
+def delete(obj, scope_selector):
+    scope, selector = scope_selector.split(":")
+
+    log_info(obj['queue'])
+    r = obj['queue'].delete(scope, selector)
+    print(colored("deleted:", "green"), ":", r)
+    log_info(obj['queue'])
+
 @cli.command()
 @click.option('-s', '--scope-selector', default="state:all")
 @click.pass_obj
@@ -463,6 +474,37 @@ def execute(obj):
     an example runner for INTEGRAL pipeline is available with data-analysis  module and \033[32moda-runner-execute\033[0m command
     """)
 
+
+@cli.command()
+@click.argument("target")
+@click.option("-m", "--module", multiple=True)
+@click.option("-a", "--assume", multiple=True)
+@click.pass_obj
+def ask(obj, target, module, assume):
+    task_data = dict(
+                object_identity=dict(
+                    assumptions=[
+                            ['ii_spectra_exract', {'request_root_node': True}]
+                        ] + [['', a] for a in assume],
+                    expected_hashe='None',
+                    factory_name=target,
+                    full_name=target,
+                    modules=[
+                            ['git', m[len("git://"):].split("/")[0], m[len("git://"):].split("/")[1]]
+                            for m in module
+                        ]
+                )
+            )
+
+    r = obj['queue'].put(
+            task_data,
+            submission_data=dict(
+                callbacks=[],
+                request_origin="cli",
+                ),
+            )
+
+
 def main():
     cli(obj={})
 
