@@ -11,12 +11,13 @@ import io
 import re
 import click
 import urllib.parse
+import pylogstash
 
 from functools import reduce
 
 from bravado.client import SwaggerClient
 
-__version__ = "0.1.40"
+__version__ = "0.1.43"
 
 try:
     import io
@@ -41,6 +42,7 @@ from dqueue.database import EventLog, TaskEntry, db, model_to_dict
 sleep_multiplier = 1
 n_failed_retries = int(os.environ.get('DQUEUE_FAILED_N_RETRY','20'))
 
+log_stasher = pylogstash.LogStasher(sep="/")
 
 def get_logger(name):
     level = getattr(logging, os.environ.get('DQUEUE_LOG_LEVEL', 'INFO'))
@@ -77,6 +79,12 @@ class CurrentTaskUnfinished(Exception):
 class TaskStolen(Exception):
     pass
 
+class DependenciesDoNotExist(Exception):
+    pass
+
+class InsertTaskAnomaly(Exception):
+    pass
+
 class Task:
     reference_task = False
 
@@ -330,7 +338,14 @@ class Queue:
     
     def try_to_unlock(self,task):
         ""
-        dependency_states=self.find_dependecies_states(task)
+
+        try:
+            dependency_states=self.find_dependecies_states(task)
+        except DependenciesDoNotExist:
+            self.log_task("task dependencies do not exist: unlocking",task,"locked")
+            self.move_task("locked", "waiting", task)
+            return dict(state="waiting", key=task.key)
+
         
 
         if all([d['state']=="done" for d in dependency_states]):
@@ -408,6 +423,7 @@ class Queue:
             else:
                 logger.error("instances_for_key is None!")
 
+
             log("sleeping for",retry_sleep_race,"attempt",ntry_race)
             time.sleep(retry_sleep_race)
             ntry_race-=1
@@ -426,6 +442,14 @@ class Queue:
             logger.debug("task entry: %s", d)
             return d
 
+        logger.info("testing it was put as corrupt...")
+        instances_for_key = self.find_task_instances(task, ["corrupt"])
+        if instances_for_key is not None:
+            logger.error("found corrupt instances for key: %s", instances_for_key)
+            #open("/tmp/problematic_entry.json", "wt").write(entry)
+            nentries = TaskEntry.delete().where(TaskEntry.key == task.key).execute(database=None)
+            logger.error("deleted entries for %s: %s", task.key, nentries)
+
         def insert_it():
             if depends_on is None:
                 self.insert_task_entry(task, "waiting")
@@ -488,9 +512,10 @@ class Queue:
         return [ model_to_dict(r) for r in EventLog.select().where(EventLog.worker_state!="unset").order_by(EventLog.timestamp.desc()).limit(1).execute(database=None) ]
 
     def get_one_task(self, update_expected_in_s, offset):
+                    # or created? was created
         select_task = TaskEntry.select(TaskEntry.key)\
                     .where( (TaskEntry.state=="waiting") & (TaskEntry.queue==self.queue) )\
-                    .order_by(TaskEntry.created)\
+                    .order_by(TaskEntry.modified)\
                     .offset(offset)\
                     .limit(1)
 
@@ -513,6 +538,7 @@ class Queue:
 
 
         entries=TaskEntry.select().where(TaskEntry.worker_id==self.worker_id,TaskEntry.state=="running").order_by(TaskEntry.modified.desc()).limit(1).execute(database=None)
+
         if len(entries)>1:
             raise Exception(f"several tasks ({len(entries)}) are running for this worker: impossible!")
 
@@ -564,8 +590,8 @@ class Queue:
 
         self.note_worker_state("ready")
 
-        max_update_expected_in_s = 3600
-        default_update_expected_in_s = 1200
+        max_update_expected_in_s = 7200
+        default_update_expected_in_s = 1800
 
         if update_expected_in_s > max_update_expected_in_s:
             logger.warning("update expected timeout %s is too long, setting to maximum %s", update_expected_in_s, max_update_expected_in_s)
@@ -576,8 +602,16 @@ class Queue:
             update_expected_in_s = default_update_expected_in_s
     
         offset = 0
+
+        tried_tasks = 0
         while True:
             self.get_one_task(update_expected_in_s, offset=offset)
+            tried_tasks += 1
+
+            if tried_tasks > 50: # TODO: HC
+                logger.warning("stopping search for task, exceeded max")
+                self.current_task = None
+                break
 
             if self.current_task is None:
                 time.sleep(1)
@@ -585,8 +619,8 @@ class Queue:
 
             worker_fit_score = self.current_task.score_worker_knowledge(worker_knowledge) # also sort TODO
             if worker_fit_score <= 0:
-                logger.warning("picked task %s has non-positive (%s) worker (%s) score: skipping",
-                        self.current_task.key, worker_fit_score, worker_knowledge)
+                logger.warning("picked task %s has non-positive (%s) worker (%s) score: skipping; tried %s current offset %s",
+                        self.current_task.key, worker_fit_score, worker_knowledge, tried_tasks, offset)
 
                 r=TaskEntry.update({
                                 TaskEntry.state:"waiting",
@@ -768,24 +802,30 @@ class Queue:
 
     def insert_task_entry(self,task,state):
         self.log_task("task created",task,state)
-
+        
+        serialized_task = task.serialize()
         log("to insert_task entry: ", dict(
              queue=self.queue,
              key=task.key,
              state=state,
              worker_id=self.worker_id,
-             task_dict_string=task.serialize(),
+             task_dict_string=serialized_task,
              created=datetime.datetime.now(),
              modified=datetime.datetime.now(),
         ))
+        
+        r = TaskEntry.select(TaskEntry.key == task.key).execute()
+        logger.error("prior to inserting, have %s", r)
 
+
+        insert_result = None
         try:
-            TaskEntry.insert(
+            insert_result = "inserted", TaskEntry.insert(
                              queue=self.queue,
                              key=task.key,
                              state=state,
                              worker_id=self.worker_id,
-                             task_dict_string=task.serialize(),
+                             task_dict_string=serialized_task,
                              created=datetime.datetime.now(),
                              modified=datetime.datetime.now(),
                             ).execute(database=None)
@@ -793,13 +833,33 @@ class Queue:
             log("task already inserted, reasserting the queue to",self.queue)
 
             # deadlock
-            TaskEntry.update(
+            insert_result = "updated", TaskEntry.update(
                                 queue=self.queue,
                             ).where(
                                 TaskEntry.key == task.key,
                             ).execute(database=None)
 
 
+        r = list(TaskEntry.select().where(TaskEntry.key == task.key).execute(database=None))
+
+        if len(r) == 0:
+            logger.error("inserted task does not exist, selecting by key %s, result %s", task.key, insert_result)
+            raise Exception(f"inserted task does not exist, selecting by key {task.key}, insert_result {insert_result}")
+        
+        if len(r) > 1:
+            logger.error("multiple tasks for key %s", task.key)
+            raise Exception(f"multiple tasks for key {task.key}")
+
+        if r[0].task_dict_string != serialized_task:
+            logger.error("insert result was %s but found mismatch between task_dict_string and stored: %s != %s; complete entry %s", 
+                    insert_result,
+                    r[0].task_dict_string, serialized_task, model_to_dict(r[0]))
+            logger.error("insert result was %s but found mismatch between task_dict_string and stored: could it be because of size limit of peewee db?")
+            raise InsertTaskAnomaly()
+
+        log("task successfully inserted")
+
+
     def find_dependecies_states(self,task):
         if task.depends_on is None:
             raise Exception("can not inspect dependecies in an independent task!")
@@ -826,7 +886,7 @@ class Queue:
             if len(dependencies[-1]['states'])==0:
                 print(("job dependencies do not exist, expecting %s"%dependency_task.key))
                 #print(dependency_task.serialize())
-                raise Exception("job dependencies do not exist, expecting %s"%dependency_task.key)
+                raise DependenciesDoNotExist("job dependencies do not exist, expecting %s"%dependency_task.key)
 
             if 'done' in dependencies[-1]['states']:
                 dependencies[-1]['state']='done'
@@ -935,7 +995,7 @@ class Queue:
         self.current_task=None
 
     def forgive_task_failures(self) -> int:
-        entries = TaskEntry.select().where(TaskEntry.state=="failed").order_by(TaskEntry.modified.desc()).limit(1).execute(database=None)
+        entries = TaskEntry.select().where(TaskEntry.state=="failed").order_by(TaskEntry.modified).limit(1).execute(database=None)
 
         if len(entries) == 0:
             logger.info("no failed tasks: will not try to forgive")
@@ -1007,7 +1067,7 @@ class Queue:
     @property
     def summary(self):
         r={}
-        for kind in "waiting","running","done","failed","locked":
+        for kind in "waiting","running","done","failed","locked", "corrupt":
             r[kind] = TaskEntry.select().where(TaskEntry.state==kind, TaskEntry.queue==self.queue).count()
            # .execute(database=None)
             #
@@ -1090,15 +1150,36 @@ class Queue:
         if state is None:
             state="undefined"
 
-        logger.info("log_task: %s:%s for %s at %s", task, task_key, message, state)
+        logger.info("log_task: task:%s task_key:%s for message:%s at state:%s", task, task_key, message, state)
+
+        # pass
+
+        log_data = dict(
+                         queue=self.queue,
+                         task_key=task_key,
+                         task_state=state,
+                         worker_id=self.worker_id,
+                   )
+
+        msg = {
+                **log_data, 
+                'origin': 'oda-node', 
+            }
+
+        try:
+            msg['message'] = json.loads(message)
+            logger.info("managed to decode message json: %s", msg['message'])
+        except Exception as e:
+            logger.warning("unable to decode message: %s - from %s", e, message)
+            msg['message'] = message
+
+        logger.info("to logstash: %s", json.dumps(pylogstash.flatten(msg, sep="/")))
+        log_stasher.log({"oda_"+k:v for k,v in msg.items()})
 
         return EventLog.insert(
-                             queue=self.queue,
-                             task_key=task_key,
-                             task_state=state,
-                             worker_id=self.worker_id,
-                             timestamp=datetime.datetime.now(),
-                             message=message,
+                            **log_data,
+                            message=message,
+                            timestamp=datetime.datetime.now(),
                         ).execute(database=None)
 
 
